var widgets = require('@jupyter-widgets/base');
var _ = require('lodash');

import ParaViewWebClient from 'paraviewweb/src/IO/WebSocket/ParaViewWebClient';
import RemoteRenderer from 'paraviewweb/src/NativeUI/Canvas/RemoteRenderer';
import SizeHelper from "paraviewweb/src/Common/Misc/SizeHelper";
import SmartConnect from 'wslink/src/SmartConnect';

var _getId = (function () {

  var cnt = 0;
  return function () {

    cnt += 1;
    return 'canvas_' + cnt;
  }
})();


export var RemoteRenderModel = widgets.DOMWidgetModel.extend({
  defaults: _.extend(widgets.DOMWidgetModel.prototype.defaults(), {
    _model_name: 'RemoteRenderModel',
    _view_name: 'RemoteRenderView',
    _model_module: 'vtk-remoterender',
    _view_module: 'vtk-remoterender',
    _model_module_version: '0.1.0',
    _view_module_version: '0.1.0',
  })
});

export var RemoteRenderView = widgets.DOMWidgetView.extend({
    render: function () {
        var parent_div = this.$el.parent().prevObject[0];
        
        var divRenderer = document.createElement('div');
        divRenderer.style.minHeight = '400px';
        divRenderer.style.minWidth = '400px';
        divRenderer.style.height = '100%';
        divRenderer.style.width = '100%';
        divRenderer.id = _getId();
        this.el.appendChild(divRenderer);

        var that = this;
        var config = { sessionURL: this.model.get('sessionURL'), secret: this.model.get('authKey') };
        console.log(config);
        var smartConnect = SmartConnect.newInstance({ config: config });
        console.log(smartConnect);
        smartConnect.onConnectionReady(function (connection) {
            var pvwClient = ParaViewWebClient.createClient(connection, ['MouseHandler', 'ViewPort', 'ViewPortImageDelivery']);
            var renderer = new RemoteRenderer(pvwClient);
            renderer.setContainer(divRenderer);
            renderer.setView(that.model.get('viewID'));
            renderer.showStats = true;
            renderer.onImageReady(function () {
                if (parent_div.style.width != '100%') {
                    parent_div.style.width = '100%';
                    renderer.resize();
                }
                console.log("We are good.");
            });
            that.model.on('change:_update', function() {
                // renderer.render(true);
                renderer.render(false);
            }, that);
            SizeHelper.onSizeChange(function () {
                renderer.resize();
            });
            SizeHelper.startListening();
        });
        smartConnect.connect();
    },
})
